/*
1. To driver 24 LEDs for breathing effect using GPIOs.

Configurations,

1. MCLK = MAXIMUM = 16Mhz
2. PWM Freq. = 5Khz
3. Lightness Levels = 24 Steps  (0---24)  OFF ---100% ON
4. Refresh Interval 100mS

So Timer intervals is about 24*5Khz = 120Khz. Set TimerA to 120Khz interrrupt.

TimerD0for 15mS interrupt;  upmode TA0CCR0 = 15mS
TimerA0 for 120Khz interupt; 
*/
#include "touch.h"

// 5K HZ, PWM
#define PWMFreq   5

#define LEDNUM  24
#define LIGHTLVL 36
#define INIT_LVL 36
#define RELOADNUM 36

#define UP  4
#define DWN 0
unsigned char direction = UP;
unsigned char sensor_timer_up_flag = 0;

unsigned char volatile *const_LED_output_reg[LEDNUM] ={&P2OUT, &P2OUT,&P2OUT,&P2OUT,&P2OUT,
                                                       &P2OUT,&P3OUT, &P3OUT,&P3OUT,&P3OUT, 
                                                       &P4OUT ,&P4OUT,&P3OUT,&P3OUT,&P2OUT,
                                                       &P2OUT,&P1OUT,&P1OUT,&P1OUT,&P1OUT, 
                                                       &P1OUT,&P1OUT,&P1OUT,&P1OUT};
const unsigned char LED_BIT[LEDNUM] = {BIT5,BIT7,BIT6,BIT0,BIT1,
                                       BIT2,BIT0,BIT1,BIT2,BIT3,
                                       BIT6,BIT7,BIT6,BIT7,BIT3,
                                       BIT4,BIT0,BIT1,BIT2,BIT3,
                                       BIT4,BIT5,BIT6,BIT7};

unsigned char LightLevel[LEDNUM] = {0,0,0,0,0,
                                    0,0,0,0,0,
                                    0,0,0,0,0,
                                    0,0,0,0,0,
                                    0,0,0,0} ;  // 24 LeDs and 24 Levels

unsigned char LeightLVLtemp[LEDNUM] = {0};  //butter for auto-decreasing

unsigned char reload_cnt = 0;
unsigned char i = 0;

extern str_sensor sensor[SENSOR_NUM];
extern unsigned press_number;
extern unsigned char sensor_name[];
extern unsigned int WheelPosition;
extern unsigned int LastPosition;


unsigned char KeyID_trigged = 0;
unsigned char key_bounce_flag = 1;
unsigned char Wheel_UnReleasedTimer = 0;   // avoid finger move out electrode some time

unsigned char volatile *p ;
void UCS_init(void) ;
void ClearIO(unsigned char iii) ;
void SetLightLevel(unsigned char TouchPosition);

void main( void )
{
  unsigned char i,j = 0;
  unsigned int measure_raw = 0;
  
  unsigned char counter = 0;
  // Stop watchdog timer to prevent time out reset
  WDTCTL = WDTPW + WDTHOLD;
  IE1 |= WDTIE;                             // Enable WDT interrupt
  
  P1OUT = 0; 
  P1DIR |= BIT0 + BIT1  + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7;

  P2OUT = 0; 
  P2DIR |= BIT0 + BIT1  + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7;

  P2SEL =0;
  P2SEL2 =0;
  
  P3OUT = 0;
  P3DIR |= BIT0 + BIT1 + BIT2 + BIT3 + BIT6+ BIT7;

  P4OUT = 0;
  P4DIR |=  BIT6+ BIT7;

  
  UCS_init();   //config MLCK = 16Mhz
  data_init();
  
  
  TA0CCTL0 = CCIE;                          // CCR0 interrupt enabled
  TA0CCR0 = 1300 ;                           // 
  TA0CTL = TASSEL_2 + MC_1 + TACLR;         // SMCLK, upmode, clear TAR
  

  TBCCTL0 = CCIE;                          // CCR0 interrupt enabled
  TBCCR0 = 200;//100;                           // 10mS interrupt 10mS = 100/ 10K
  TBCTL = TASSEL_1+ MC_1 + TACLR;        // ACLK, upmode, clear TDR

  reload_cnt = RELOADNUM;
     
  for(counter = 0 ; counter < LEDNUM ; counter ++) {
    
    LightLevel[counter] =0;
    LeightLVLtemp[counter] = 0;
  }  
  direction = UP;
 
  while(1) {    
		if(!Wheel_UnReleasedTimer)
			Wheel_UnReleasedTimer=WheelReleaseMax;
		else
			Wheel_UnReleasedTimer--;
		measure_raw = 0;
		for (i = 0; i < (SENSOR_NUM); i++) {
			measure_raw = sensor_scan(i);
			MEASURECNT = (unsigned int) filter(i, measure_raw, FILTER_FACTOR); //filter for raw data

			if (MEASURECNT >= BASELINE) { //tracking baseline,with cap decrease(cnt increase)
//add baseline tracking algorithm here, it depends on the touch mechanical design
			} else { 
				DELTACNT = BASELINE - MEASURECNT;
				if (DELTACNT < (THRESHOLD >> 2)) { //if delta < 1/4 Threshold, then update baseline ,
//add baseline tracking algorithm here, it depends on the touch mechanical design
					PRESSCNT = 0;
				} else { //if delta > trheshold ,counting ++
					if (DELTACNT > THRESHOLD) {
						if (PRESSCNT > 200)
							PRESSCNT = PRESSEDENOUGH; // prevent from too much times.
						PRESSCNT++;
					} else
						//or do-nothing ,assuming noises are too high
						PRESSCNT = 0; //do nothing           
				}
			}
		}
		WheelPosition = Wheel_Calulate();

#ifdef INTERPOLATION
		if((WheelPosition!=ILLEGAL_SLIDER_WHEEL_POSITION) & (WheelPosition < (Channel_Resolution * Slider_Wheel_Num)))
		{
                      if(TBCCR0 == 200)                        
                      {
                        TBCCR0 = 100;//100;                           // 10mS interrupt 10mS = 100/ 10K
                      }
			Wheel_UnReleasedTimer=WheelReleaseMax;
			if(LastPosition!=ILLEGAL_SLIDER_WHEEL_POSITION)
			{
				if(WheelPosition>(LastPosition+1))
				{
					if((WheelPosition-LastPosition)< InterpolationThreshold)
					{
						for (j=1;j<=(WheelPosition-LastPosition+1);j++)
						{
								SetLightLevel(LastPosition+j);
						}
					}
					else
					{
						// pass 0 point by unti_clock direction, 2->1->0->17->16
						if((WheelPosition>((Channel_Resolution * Slider_Wheel_Num-1)-(InterpolationThreshold/2))) & (LastPosition<(InterpolationThreshold/2))) // pass 0 point by unti_clock direction
						{
							if(LastPosition)
							{
								for (j=1;j<=LastPosition;j++)
								{
										SetLightLevel(LastPosition-j);
								}
							}
							if(WheelPosition<(Channel_Resolution * Slider_Wheel_Num-1))
							{
								for (j=0;j<=((Channel_Resolution * Slider_Wheel_Num-1)-WheelPosition);j++)
								{
										SetLightLevel((Channel_Resolution * Slider_Wheel_Num-1)-j);
								}
							}
							else
							{
								SetLightLevel(WheelPosition);
							}
						}
						else
							SetLightLevel(WheelPosition);
					}
				}
				else if (WheelPosition<(LastPosition-1))
				{
					if((LastPosition-WheelPosition)< InterpolationThreshold)
					{
						for (j=1;j<(LastPosition-WheelPosition+1);j++)
						{
								SetLightLevel(LastPosition-j);
						}
					}
					else
					{
						// pass 0 point by clock direction,  16->17->0->1->2
						if((LastPosition>((Channel_Resolution * Slider_Wheel_Num-1)-(InterpolationThreshold/2))) & (WheelPosition<(InterpolationThreshold/2)))
						{

							if(LastPosition<(Channel_Resolution * Slider_Wheel_Num-1))
							{
								for (j=((Channel_Resolution * Slider_Wheel_Num-1)-LastPosition);j>0;j--)
								{
										SetLightLevel((Channel_Resolution * Slider_Wheel_Num-1)-(j-1));
								}
							}

							for (j=0;j<=WheelPosition;j++)
							{
								SetLightLevel(j);
							}

						}
						else
							SetLightLevel(WheelPosition);
					}
				}
				else
					SetLightLevel(WheelPosition);
			}
			else
				SetLightLevel(WheelPosition);
		}

		if(Wheel_UnReleasedTimer)
			LastPosition = WheelPosition;
#else
		SetLightLevel(WheelPosition);
#endif
  
  }

}

void SetLightLevel(unsigned char TouchPosition)
{
        if(TouchPosition < LEDNUM)
        {
           if ( LightLevel[TouchPosition] < INIT_LVL )
             LightLevel[TouchPosition] = INIT_LVL;
           else {
              LightLevel[TouchPosition] += 1;
              if ( LightLevel[TouchPosition] > LIGHTLVL )
              LightLevel[TouchPosition] = LIGHTLVL ;
           }
        }
}


void UCS_init(void) 

{
	WDTCTL = WDTPW + WDTHOLD; // Stop watchdog timer to prevent time out reset
	BCSCTL1 = CALBC1_16MHZ;
	DCOCTL = CALDCO_16MHZ;
	BCSCTL3 |= LFXT1S_2; // LFXT1 = VLO
	WDTCTL = WDT_ADLY_1000; //Aclk = VLO = 12Khz, to source WDT timer
    
}





void  ClearIO(unsigned char iii)   //turn IO to 0;
{
   p = const_LED_output_reg[iii];
   *p &= ~LED_BIT[iii]; 
}



// Timer0 A0 interrupt service routine
#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR(void)
{
  __enable_interrupt();
  if(reload_cnt) {
    reload_cnt--;
    
    for (i = 0; i < LEDNUM ; i ++) {
      
      if ( LeightLVLtemp[i] >= 1)
         LeightLVLtemp[i] --;  
      else  { 
        p = const_LED_output_reg[i];
        *p &= ~LED_BIT[i];     
      }    
    }  
    
  }
  
  else  { // reload
    
    reload_cnt = RELOADNUM;
    for (i = 0; i < LEDNUM ; i ++) {
       LeightLVLtemp[i] = LightLevel[i];
       if(LeightLVLtemp[i]) {
          p = const_LED_output_reg[i];
         *p |= LED_BIT[i];  
       }
       else  {
           p = const_LED_output_reg[i];
           *p &= ~LED_BIT[i]; 
         
       }              
  }
  }
  
  if(sensor_timer_up_flag) {
    LPM0_EXIT;
    sensor_timer_up_flag = 0;
  }
  
}



// TimerB interrupt service routine,every 10mS for lightness decrease
#pragma vector=TIMERB0_VECTOR
__interrupt void TIMERB_ISR(void)
{
  __enable_interrupt();
  for (i = 0; i < LEDNUM ; i ++) {
    
    if(direction == DWN) {
    if (LightLevel[i])
      LightLevel[i] --;
    else
       LightLevel[i] = 0;
    }
    else {
       if (LightLevel[i] < LIGHTLVL ) 
         LightLevel[i] ++;
       else {
         direction = DWN;
       } 
    }
}

  if(sensor_timer_up_flag) {
    LPM0_EXIT ;
    sensor_timer_up_flag = 0;
  }  
}

// WatchDog interrupt service routine, for touch scan
#pragma vector=WDT_VECTOR
__interrupt void WDT_ISR(void)
{
     TA1CTL = 0; 	
     LPM0_EXIT;
     sensor_timer_up_flag = 1;
}


